@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** Licensed under the Apache License, Version 2.0 (the "License");
 ** you may not use this file except in compliance with the License.
@@ -20,17 +20,18 @@ package dbconn
 import (
 	"context"
 	"database/sql"
+	"fmt"
 	"net/url"
 	"path/filepath"
 	"strconv"
 	"sync"
 	"time"
 
-	"git.zabbix.com/ap/mssql/plugin/handlers"
-	"git.zabbix.com/ap/mssql/plugin/params"
-	"git.zabbix.com/ap/plugin-support/errs"
-	"git.zabbix.com/ap/plugin-support/log"
-	"git.zabbix.com/ap/plugin-support/uri"
+	"golang.zabbix.com/plugin/mssql/plugin/handlers"
+	"golang.zabbix.com/plugin/mssql/plugin/params"
+	"golang.zabbix.com/sdk/errs"
+	"golang.zabbix.com/sdk/log"
+	"golang.zabbix.com/sdk/uri"
 )
 
 var (
@@ -137,17 +138,13 @@ func (c *ConnCollection) Close() {
 	}
 }
 
-//
 //nolint:gocritic // need conf by value.
 func (c *ConnCollection) get(
 	ctx context.Context,
 	conf connConfig,
 ) (*sql.DB, error) {
-	c.mu.Lock()
-	defer c.mu.Unlock()
-
-	conn, ok := c.conns[conf]
-	if ok {
+	conn := c.getConn(conf)
+	if conn != nil {
 		return conn, nil
 	}
 
@@ -156,9 +153,47 @@ func (c *ConnCollection) get(
 		return nil, errs.Wrap(err, "failed to create conn")
 	}
 
+	return c.setConn(conf, conn), nil
+}
+
+// getConn concurrent connections cache getter.
+func (c *ConnCollection) getConn(conf connConfig) *sql.DB { //nolint:gocritic
+	c.mu.Lock()
+	defer c.mu.Unlock()
+
+	conn, ok := c.conns[conf]
+	if !ok {
+		return nil
+	}
+
+	return conn
+}
+
+// setConn concurrent connections cache setter.
+//
+// Returns the cached connection. If the provider connection is already present
+// in cache, it is closed.
+//
+//nolint:gocritic
+func (c *ConnCollection) setConn(
+	conf connConfig,
+	conn *sql.DB,
+) *sql.DB {
+	c.mu.Lock()
+	defer c.mu.Unlock()
+
+	existingConn, ok := c.conns[conf]
+	if ok {
+		defer conn.Close() //nolint:errcheck
+
+		c.logr.Debugf("Closed redundant connection: %s", conf.URI)
+
+		return existingConn
+	}
+
 	c.conns[conf] = conn
 
-	return conn, nil
+	return conn
 }
 
 //nolint:gocyclo,cyclop
@@ -183,7 +218,7 @@ func (c *ConnCollection) newConn(
 	)
 
 	connURI, err := uri.NewWithCreds(
-		conf.URI, conf.User, conf.Password, params.URIDefaults,
+		conf.URI, conf.User, conf.Password, &uri.Defaults{Scheme: params.URIDefaults.Scheme},
 	)
 	if err != nil {
 		return nil, errs.Wrap(err, "failed to set URI defaults")
@@ -194,6 +229,12 @@ func (c *ConnCollection) newConn(
 		return nil, errs.Wrap(err, "failed to parse URI")
 	}
 
+	if connURI.Port() == "" && connURI.Path() == "" {
+		u.Host = fmt.Sprintf("%s:%s", u.Hostname(), params.URIDefaults.Port)
+	}
+
+	u.Path = connURI.Path()
+
 	queryParams := u.Query()
 	queryParams.Add("app name", "Zabbix agent 2 MSSQL plugin")
 	queryParams.Add("keepAlive", strconv.Itoa(c.keepAlive))
